@@ -1,4 +1,4 @@
-{-# LANGUAGE RecordWildCards, ViewPatterns #-}
+{-# LANGUAGE RecordWildCards, ViewPatterns, ScopedTypeVariables #-}
 
 module Development.Ninja.All(runNinja) where
 
@@ -23,15 +23,20 @@ import System.Info.Extra
 import Prelude
 
 -- Internal imports
+import General.Extra(removeFile_)
 import General.Timing(addTiming)
-import Development.Shake.ByteString(filepathNormalise, parseMakefile)
-import Development.Shake.Errors(errorStructured)
-import Development.Shake.Rules.File(needBS, neededBS)
-import Development.Shake.Rules.OrderOnly(orderOnlyBS)
+import General.Makefile(parseMakefile)
+import Development.Shake.Internal.FileName(filepathNormalise, fileNameFromString)
+import Development.Shake.Internal.FileInfo(getFileInfo)
+import Development.Shake.Internal.Errors(errorStructured)
+import Development.Shake.Internal.Rules.File(needBS, neededBS)
+import Development.Shake.Internal.Rules.OrderOnly(orderOnlyBS)
 
 
-runNinja :: FilePath -> [String] -> Maybe String -> IO (Maybe (Rules ()))
-runNinja file args (Just "compdb") = do
+-- | Given the Ninja source file, a list of file arguments, a tool name.
+--   Return a bool if you should restart and the rules.
+runNinja :: IO () -> FilePath -> [String] -> Maybe String -> IO (Maybe (Rules ()))
+runNinja restart file args (Just "compdb") = do
     dir <- getCurrentDirectory
     Ninja{..} <- parse file =<< newEnv
     rules <- return $ Map.fromList [r | r <- rules, BS.unpack (fst r) `elem` args]
@@ -51,26 +56,34 @@ runNinja file args (Just "compdb") = do
     putStr $ printCompDb xs
     return Nothing
 
-runNinja file args (Just x) = errorIO $ "Unknown tool argument, expected 'compdb', got " ++ x
+runNinja restart file args (Just x) = errorIO $ "Unknown tool argument, expected 'compdb', got " ++ x
 
-runNinja file args tool = do
+runNinja restart file args tool = do
     addTiming "Ninja parse"
     ninja@Ninja{..} <- parse file =<< newEnv
     return $ Just $ do
-        needDeps <- return $ needDeps ninja -- partial application
         phonys <- return $ Map.fromList phonys
+        needDeps <- return $ needDeps ninja phonys -- partial application
         singles <- return $ Map.fromList $ map (first filepathNormalise) singles
         multiples <- return $ Map.fromList [(x,(xs,b)) | (xs,b) <- map (first $ map filepathNormalise) multiples, x <- xs]
         rules <- return $ Map.fromList rules
         pools <- fmap Map.fromList $ forM ((BS.pack "console",1):pools) $ \(name,depth) ->
             (,) name <$> newResource (BS.unpack name) depth
 
-        action $ needBS $ concatMap (resolvePhony phonys) $
-            if not $ null args then map BS.pack args
-            else if not $ null defaults then defaults
-            else Map.keys singles ++ Map.keys multiples
+        action $ do
+            -- build the .ninja files, if they change, restart the build
+            before <- liftIO $ mapM (getFileInfo . fileNameFromString) sources
+            need sources
+            after <- liftIO $ mapM (getFileInfo . fileNameFromString) sources
+            if before /= after then
+                runAfter restart
+             else
+                needBS $ concatMap (resolvePhony phonys) $
+                    if not $ null args then map BS.pack args
+                    else if not $ null defaults then defaults
+                    else Map.keys singles ++ Map.keys multiples
 
-        (\x -> (map BS.unpack . fst) <$> Map.lookup (BS.pack x) multiples) &?> \out -> let out2 = map BS.pack out in
+        (\x -> map BS.unpack . fst <$> Map.lookup (BS.pack x) multiples) &?> \out -> let out2 = map BS.pack out in
             build needDeps phonys rules pools out2 $ snd $ multiples Map.! head out2
 
         (flip Map.member singles . BS.pack) ?> \out -> let out2 = BS.pack out in
@@ -137,8 +150,8 @@ build needDeps phonys rules pools out build@Build{..} = do
                     -- when (deps == "gcc") $ liftIO $ removeFile depfile
 
 
-needDeps :: Ninja -> Build -> [Str] -> Action ()
-needDeps Ninja{..} = \build xs -> do -- eta reduced so 'builds' is shared
+needDeps :: Ninja -> Map.HashMap Str [Str] -> Build -> [Str] -> Action ()
+needDeps Ninja{..} phonysMp = \build xs -> do -- eta reduced so 'builds' is shared
     opts <- getShakeOptions
     if isNothing $ shakeLint opts then needBS xs else do
         neededBS xs
@@ -149,9 +162,11 @@ needDeps Ninja{..} = \build xs -> do -- eta reduced so 'builds' is shared
         let bad = xs `difference` allDependencies build
         case bad of
             [] -> return ()
-            x:_ -> liftIO $ errorStructured
-                "Lint checking error - file in deps is generated and not a pre-dependency"
-                [("File", Just $ BS.unpack x)]
+            xs -> liftIO $ errorStructured
+                ("Lint checking error - " ++
+                    (if length xs == 1 then "file in deps is" else "files in deps are") ++
+                    " generated and not a pre-dependency")
+                [("File", Just $ BS.unpack x) | x <- xs]
                 ""
     where
         builds :: Map.HashMap FileStr Build
@@ -172,7 +187,7 @@ needDeps Ninja{..} = \build xs -> do -- eta reduced so 'builds' is shared
         allDependencies rule = f Set.empty [] [rule]
             where
                 f seen [] [] = []
-                f seen [] (x:xs) = f seen (map filepathNormalise $ depsNormal x ++ depsImplicit x ++ depsOrderOnly x) xs
+                f seen [] (x:xs) = f seen (map filepathNormalise $ concatMap (resolvePhony phonysMp) $ depsNormal x ++ depsImplicit x ++ depsOrderOnly x) xs
                 f seen (x:xs) rest | x `Set.member` seen = f seen xs rest
                                    | otherwise = x : f (Set.insert x seen) xs (maybeToList (Map.lookup x builds) ++ rest)
 
@@ -184,7 +199,7 @@ applyRspfile env act = do
     if rspfile == "" then
         act
      else
-        flip actionFinally (ignore $ removeFile rspfile) $ do
+        flip actionFinally (removeFile_ rspfile) $ do
             liftIO $ BS.writeFile rspfile rspfile_content
             act
 
