@@ -3,10 +3,11 @@
 -- {-# OPTIONS_GHC -ddump-simpl #-}
 
 -- | Lexing is a slow point, the code below is optimised
-module Development.Ninja.Lexer(Lexeme(..), lexer, lexerFile) where
+module Development.Ninja.Lexer(Lexeme(..), lexerFile) where
 
 import Control.Applicative
 import Data.Tuple.Extra
+import Data.Char
 import qualified Data.ByteString.Char8 as BS
 import qualified Data.ByteString.Unsafe as BS
 import Development.Ninja.Type
@@ -25,11 +26,11 @@ newtype Str0 = Str0 Str -- null terminated
 
 type S = Ptr Word8
 
-chr :: S -> Char
-chr x = Internal.w2c $ unsafePerformIO $ peek x
+char :: S -> Char
+char x = Internal.w2c $ unsafePerformIO $ peek x
 
-inc :: S -> S
-inc x = x `plusPtr` 1
+next :: S -> S
+next x = x `plusPtr` 1
 
 {-# INLINE dropWhile0 #-}
 dropWhile0 :: (Char -> Bool) -> Str0 -> Str0
@@ -49,8 +50,8 @@ break0 f (Str0 bs) = (BS.unsafeTake i bs, Str0 $ BS.unsafeDrop i bs)
             return $! Ptr end `minusPtr` start
 
         go s@(Ptr a) | c == '\0' || f c = a
-                     | otherwise = go (inc s)
-            where c = chr s
+                     | otherwise = go (next s)
+            where c = char s
 
 {-# INLINE break00 #-}
 -- The predicate must return true for '\0'
@@ -63,8 +64,8 @@ break00 f (Str0 bs) = (BS.unsafeTake i bs, Str0 $ BS.unsafeDrop i bs)
             return $! Ptr end `minusPtr` start
 
         go s@(Ptr a) | f c = a
-                     | otherwise = go (inc s)
-            where c = chr s
+                     | otherwise = go (next s)
+            where c = char s
 
 head0 :: Str0 -> Char
 head0 (Str0 x) = Internal.w2c $ BS.unsafeHead x
@@ -95,7 +96,7 @@ data Lexeme
       deriving Show
 
 isVar, isVarDot :: Char -> Bool
-isVar x = x == '-' || x == '_' || (x >= 'a' && x <= 'z') || (x >= 'A' && x <= 'Z') || (x >= '0' && x <= '9')
+isVar x = x == '-' || x == '_' || isAsciiLower x || isAsciiUpper x || isDigit x
 isVarDot x = x == '.' || isVar x
 
 endsDollar :: Str -> Bool
@@ -120,16 +121,16 @@ lexerLoop c_x | (c,x) <- list0 c_x = case c of
     '\n' -> lexerLoop x
     ' ' -> lexBind $ dropSpace x
     '#' -> lexerLoop $ dropWhile0 (/= '\n') x
-    'b' | Just x <- strip "uild " x -> lexBuild $ dropSpace x
-    'r' | Just x <- strip "ule " x -> lexRule $ dropSpace x
-    'd' | Just x <- strip "efault " x -> lexDefault $ dropSpace x
-    'p' | Just x <- strip "ool " x -> lexPool $ dropSpace x
-    'i' | Just x <- strip "nclude " x -> lexInclude $ dropSpace x
-    's' | Just x <- strip "ubninja " x -> lexSubninja $ dropSpace x
+    'b' | Just x <- strip "uild " x -> lexBuild x
+    'r' | Just x <- strip "ule " x -> lexRule x
+    'd' | Just x <- strip "efault " x -> lexDefault x
+    'p' | Just x <- strip "ool " x -> lexPool x
+    'i' | Just x <- strip "nclude " x -> lexInclude x
+    's' | Just x <- strip "ubninja " x -> lexSubninja x
     '\0' -> []
     _ -> lexDefine c_x
     where
-        strip str (Str0 x) = if b `BS.isPrefixOf` x then Just $ Str0 $ BS.drop (BS.length b) x else Nothing
+        strip str (Str0 x) = if b `BS.isPrefixOf` x then Just $ dropSpace $ Str0 $ BS.drop (BS.length b) x else Nothing
             where b = BS.pack str
 
 lexBind :: Str0 -> [Lexeme]
@@ -143,7 +144,7 @@ lexBind c_x | (c,x) <- list0 c_x = case c of
 lexBuild :: Str0 -> [Lexeme]
 lexBuild x
     | (outputs,x) <- lexxExprs True x
-    , (rule,x) <- span0 isVarDot $ dropSpace x
+    , (rule,x) <- span0 isVarDot $ jumpCont $ dropSpace x
     , (deps,x) <- lexxExprs False $ dropSpace x
     = LexBuild outputs rule deps : lexerLoop x
 
@@ -162,8 +163,8 @@ lexDefine = lexxBind LexDefine
 lexxBind :: (Str -> Expr -> Lexeme) -> Str0 -> [Lexeme]
 lexxBind ctor x
     | (var,x) <- span0 isVarDot x
-    , ('=',x) <- list0 $ dropSpace x
-    , (exp,x) <- lexxExpr False False $ dropSpace x
+    , ('=',x) <- list0 $ jumpCont $ dropSpace x
+    , (exp,x) <- lexxExpr False False $ jumpCont $ dropSpace x
     = ctor var exp : lexerLoop x
 lexxBind _ x = error $ show ("parse failed when parsing binding", take0 100 x)
 
@@ -181,15 +182,16 @@ lexxName ctor x
 lexxExprs :: Bool -> Str0 -> ([Expr], Str0)
 lexxExprs stopColon x = case lexxExpr stopColon True x of
     (a,c_x) | c <- head0 c_x, x <- tail0 c_x -> case c of
-        ' ' -> first (a:) $ lexxExprs stopColon $ dropSpace x
-        ':' | stopColon -> ([a], x)
+        ' ' -> add a $ lexxExprs stopColon $ dropSpace x
+        ':' | stopColon -> new a x
         _ | stopColon -> error "expected a colon"
-        '\r' -> a $: dropN x
-        '\n' -> a $: x
-        '\0' -> a $: c_x
+        '\r' -> new a $ dropN x
+        '\n' -> new a x
+        '\0' -> new a c_x
     where
-        Exprs [] $: x = ([], x)
-        a $: x = ([a], x)
+        new a x = add a ([], x)
+        add (Exprs []) x = x
+        add a (as,x) = (a:as,x)
 
 
 {-# NOINLINE lexxExpr #-}
@@ -219,6 +221,15 @@ lexxExpr stopColon stopSpace = first exprs . f
             _ | (name,x) <- span0 isVar c_x, not $ BS.null name -> Var name $: f x
             _ -> error "Unexpect $ followed by unexpected stuff"
 
+jumpCont :: Str0 -> Str0
+jumpCont o
+    | '$' <- head0 o
+    , let x = tail0 o
+    = case head0 x of
+        '\n' -> dropSpace $ tail0 x
+        '\r' -> dropSpace $ dropN $ tail0 x
+        _ -> o
+    | otherwise = o
 
 splitLineCont :: Str0 -> (Str, Str0)
 splitLineCont x = first BS.concat $ f x
