@@ -1,3 +1,4 @@
+{-# LANGUAGE TypeFamilies, ConstraintKinds #-}
 
 -- | This module is used for defining Shake build systems. As a simple example of a Shake build system,
 --   let us build the file @result.tar@ from the files listed by @result.txt@:
@@ -30,16 +31,108 @@
 --   To find out more:
 --
 -- * The user manual contains a longer example and background information on how to use Shake
---   <http://www.shakebuild.com/manual>.
+--   <https://www.shakebuild.com/manual>.
 --
--- * The home page has links to additional information <http://www.shakebuild.com/>, including
+-- * The home page has links to additional information <https://www.shakebuild.com/>, including
 --   a mailing list.
 --
 -- * The theory behind Shake is described in an ICFP 2012 paper,
---   <http://community.haskell.org/~ndm/downloads/paper-shake_before_building-10_sep_2012.pdf Shake Before Building -- Replacing Make with Haskell>.
---   The <http://www.youtube.com/watch?v=xYCPpXVlqFM associated talk> forms a short overview of Shake .
---
---   /== WRITING A BUILD SYSTEM ==============================/
+--   <https://ndmitchell.com/downloads/paper-shake_before_building-10_sep_2012.pdf Shake Before Building -- Replacing Make with Haskell>.
+--   The <https://www.youtube.com/watch?v=xYCPpXVlqFM associated talk> forms a short overview of Shake .
+module Development.Shake(
+    -- * Writing a build system
+    -- $writing
+
+    -- * GHC build flags
+    -- $flags
+
+    -- * Core
+    shake,
+    shakeOptions,
+    Rules, action, withoutActions, alternatives, priority,
+    Action, traced,
+    liftIO, actionOnException, actionFinally, runAfter,
+    ShakeException(..),
+    -- * Configuration
+    ShakeOptions(..), Rebuild(..), Lint(..), Change(..),
+    getShakeOptions, getShakeOptionsRules, getHashedShakeVersion,
+    getShakeExtra, getShakeExtraRules, addShakeExtra,
+    -- ** Command line
+    shakeArgs, shakeArgsWith, shakeArgsOptionsWith, shakeOptDescrs,
+    -- ** Progress reporting
+    Progress(..), progressSimple, progressDisplay, progressTitlebar, progressProgram, getProgress,
+    -- ** Verbosity
+    Verbosity(..), getVerbosity, putLoud, putNormal, putQuiet, withVerbosity, quietly,
+    -- * Running commands
+    command, command_, cmd, cmd_, unit,
+    Stdout(..), Stderr(..), Stdouterr(..), Exit(..), Process(..), CmdTime(..), CmdLine(..),
+    CmdResult, CmdString, CmdOption(..),
+    addPath, addEnv,
+    -- * Explicit parallelism
+    parallel, forP, par,
+    -- * Utility functions
+    copyFile', copyFileChanged,
+    readFile', readFileLines,
+    writeFile', writeFileLines, writeFileChanged,
+    removeFiles, removeFilesAfter,
+    withTempFile, withTempDir,
+    withTempFileWithin, withTempDirWithin,
+    -- * File rules
+    need, want, (%>), (|%>), (?>), phony, (~>), phonys,
+    (&%>), (&?>),
+    orderOnly, orderOnlyAction,
+    FilePattern, (?==), (<//>), filePattern,
+    needed, trackRead, trackWrite, trackAllow,
+    -- * Directory rules
+    doesFileExist, doesDirectoryExist, getDirectoryContents, getDirectoryFiles, getDirectoryDirs,
+    getDirectoryFilesIO,
+    -- * Environment rules
+    getEnv, getEnvWithDefault,
+    -- * Oracle rules
+    ShakeValue, RuleResult, addOracle, addOracleCache, askOracle,
+    -- * Special rules
+    alwaysRerun,
+    -- * Resources
+    Resource, newResource, newResourceIO, withResource, withResources,
+    newThrottle, newThrottleIO,
+    unsafeExtraThread,
+    -- * Cache
+    newCache, newCacheIO,
+    -- * Batching
+    needHasChanged,
+    resultHasChanged,
+    batch,
+    -- * Deprecated
+    (*>), (|*>), (&*>),
+    (**>), (*>>), (?>>),
+    askOracleWith
+    ) where
+
+import Prelude(Maybe, FilePath) -- Since GHC 7.10 duplicates *>
+
+-- I would love to use module export in the above export list, but alas Haddock
+-- then shows all the things that are hidden in the docs, which is terrible.
+import Control.Monad.IO.Class
+import Development.Shake.Internal.Value
+import Development.Shake.Internal.Options
+import Development.Shake.Internal.Core.Run
+import Development.Shake.Internal.Core.Rules
+import Development.Shake.Internal.Derived
+import Development.Shake.Internal.Errors
+import Development.Shake.Internal.Progress
+import Development.Shake.Internal.Args
+import Development.Shake.Internal.Shake
+
+import Development.Shake.Command
+import Development.Shake.Internal.FilePattern
+import Development.Shake.Internal.Rules.Directory
+import Development.Shake.Internal.Rules.File
+import Development.Shake.Internal.Rules.Files
+import Development.Shake.Internal.Rules.Oracle
+import Development.Shake.Internal.Rules.OrderOnly
+import Development.Shake.Internal.Rules.Rerun
+
+-- $writing
 --
 --   When writing a Shake build system, start by defining what you 'want', then write rules
 --   with '%>' to produce the results. Before calling 'cmd' you should ensure that any files the command
@@ -65,19 +158,19 @@
 --   'getDirectoryFiles' to track information other than just the contents of files. For information in the environment
 --   that you suspect will change regularly (perhaps @ghc@ version number), either write the information to
 --   a file with 'alwaysRerun' and 'writeFileChanged', or use 'addOracle'.
---
---   /== GHC BUILD FLAGS ==============================/
+
+-- $flags
 --
 --   For large build systems the choice of GHC flags can have a significant impact. We recommend:
 --
--- > ghc --make MyBuildSystem -rtsopts -with-rtsopts=-I0
+-- > ghc --make MyBuildSystem -threaded -rtsopts "-with-rtsopts=-I0 -qg -qb"
 --
 --   * @-rtsopts@: Allow the setting of further GHC options at runtime.
 --
 --   * @-I0@: Disable idle garbage collection, to avoid frequent unnecessary garbage collection, see
---     <http://stackoverflow.com/questions/34588057/why-does-shake-recommend-disabling-idle-garbage-collection/ a full explanation>.
+--     <https://stackoverflow.com/questions/34588057/why-does-shake-recommend-disabling-idle-garbage-collection/ a full explanation>.
 --
---   * With GHC 7.6 and before, omit @-threaded@: <http://ghc.haskell.org/trac/ghc/ticket/7646 GHC bug 7646>
+--   * With GHC 7.6 and before, omit @-threaded@: <https://ghc.haskell.org/trac/ghc/ticket/7646 GHC bug 7646>
 --     can cause a race condition in build systems that write files then read them. Omitting @-threaded@ will
 --     still allow your 'cmd' actions to run in parallel, so most build systems will still run in parallel.
 --
@@ -85,85 +178,6 @@
 --     to disable parallel garbage collection. Parallel garbage collection in Shake
 --     programs typically goes slower than sequential garbage collection, while occupying many cores that
 --     could be used for running system commands.
---
---   /Acknowledgements/: Thanks to Austin Seipp for properly integrating the profiling code.
-module Development.Shake(
-    -- * Core
-    shake,
-    shakeOptions,
-    Rules, action, withoutActions, alternatives, priority,
-    Action, traced,
-    liftIO, actionOnException, actionFinally,
-    ShakeException(..),
-    -- * Configuration
-    ShakeOptions(..), Assume(..), Lint(..), Change(..), getShakeOptions, getHashedShakeVersion,
-    -- ** Command line
-    shakeArgs, shakeArgsWith, shakeOptDescrs,
-    -- ** Progress reporting
-    Progress(..), progressSimple, progressDisplay, progressTitlebar, progressProgram, getProgress,
-    -- ** Verbosity
-    Verbosity(..), getVerbosity, putLoud, putNormal, putQuiet, withVerbosity, quietly,
-    -- * Running commands
-    command, command_, cmd, unit,
-    Stdout(..), Stderr(..), Stdouterr(..), Exit(..), CmdTime(..), CmdLine(..),
-    CmdResult, CmdString, CmdOption(..),
-    addPath, addEnv,
-    -- * Explicit parallelism
-    parallel, forP, par,
-    -- * Utility functions
-    copyFile', copyFileChanged,
-    readFile', readFileLines,
-    writeFile', writeFileLines, writeFileChanged,
-    removeFiles, removeFilesAfter,
-    withTempFile, withTempDir,
-    -- * File rules
-    need, want, (%>), (|%>), (?>), phony, (~>), phonys,
-    (&%>), (&?>),
-    orderOnly, orderOnlyAction,
-    FilePattern, (?==), (<//>),
-    needed, trackRead, trackWrite, trackAllow,
-    -- * Directory rules
-    doesFileExist, doesDirectoryExist, getDirectoryContents, getDirectoryFiles, getDirectoryDirs,
-    getDirectoryFilesIO,
-    -- * Environment rules
-    getEnv, getEnvWithDefault,
-    -- * Oracle rules
-    ShakeValue, addOracle, askOracle, askOracleWith,
-    -- * Special rules
-    alwaysRerun,
-    -- * Resources
-    Resource, newResource, newResourceIO, withResource, withResources,
-    newThrottle, newThrottleIO,
-    unsafeExtraThread,
-    -- * Cache
-    newCache, newCacheIO,
-    -- * Deprecated
-    (*>), (|*>), (&*>),
-    (**>), (*>>), (?>>),
-    system', systemCwd, systemOutput
-    ) where
-
-import Prelude(Maybe, FilePath) -- Since GHC 7.10 duplicates *>
-
--- I would love to use module export in the above export list, but alas Haddock
--- then shows all the things that are hidden in the docs, which is terrible.
-import Control.Monad.IO.Class
-import Development.Shake.Types
-import Development.Shake.Core hiding (trackAllow)
-import Development.Shake.Derived
-import Development.Shake.Errors
-import Development.Shake.Progress
-import Development.Shake.Args
-import Development.Shake.Shake
-
-import Development.Shake.Command
-import Development.Shake.Rules.Directory
-import Development.Shake.Rules.File
-import Development.Shake.FilePattern
-import Development.Shake.Rules.Files
-import Development.Shake.Rules.Oracle
-import Development.Shake.Rules.OrderOnly
-import Development.Shake.Rules.Rerun
 
 
 ---------------------------------------------------------------------
@@ -200,3 +214,12 @@ infix 1 *>, |*>, &*>
 -- | /Deprecated:/ Alias for '&%>'.
 (&*>) :: [FilePattern] -> ([FilePath] -> Action ()) -> Rules ()
 (&*>) = (&%>)
+
+
+---------------------------------------------------------------------
+-- DEPRECATED SINCE 0.16.1, NOV 2017
+
+-- | /Depreciated:/ Replace @'askOracleWith' q a@ by @'askOracle' q@
+--   since the 'RuleResult' type family now fixes the result type.
+askOracleWith :: (RuleResult q ~ a, ShakeValue q, ShakeValue a) => q -> a -> Action a
+askOracleWith question _ = askOracle question
