@@ -1,4 +1,11 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE FlexibleInstances, TypeSynonymInstances, TypeOperators, ScopedTypeVariables, NamedFieldPuns #-}
+{-# LANGUAGE GADTs, GeneralizedNewtypeDeriving #-}
+
+#if __GLASGOW_HASKELL__ < 710
+{-# LANGUAGE OverlappingInstances #-}
+#endif
+
 
 -- | This module provides functions for calling command line programs, primarily
 --   'command' and 'cmd'. As a simple example:
@@ -10,20 +17,21 @@
 --   The functions from this module are now available directly from "Development.Shake".
 --   You should only need to import this module if you are using the 'cmd' function in the 'IO' monad.
 module Development.Shake.Command(
-    command, command_, cmd, unit, CmdArguments, (:->),
+    command, command_, cmd, cmd_, unit, CmdArgument(..), CmdArguments(..), IsCmdArgument(..), (:->),
     Stdout(..), Stderr(..), Stdouterr(..), Exit(..), Process(..), CmdTime(..), CmdLine(..),
     CmdResult, CmdString, CmdOption(..),
     addPath, addEnv,
     ) where
 
 import Data.Tuple.Extra
-import Control.Exception.Extra
 import Control.Monad.Extra
 import Control.Monad.IO.Class
 import Data.Char
 import Data.Either.Extra
 import Data.List.Extra
 import Data.Maybe
+import Data.Semigroup (Semigroup)
+import Data.Monoid
 import System.Directory
 import System.Environment.Extra
 import System.Exit
@@ -34,17 +42,18 @@ import System.Time.Extra
 import System.IO.Unsafe(unsafeInterleaveIO)
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.Lazy.Char8 as LBS
+import General.Extra
 import General.Process
 import Control.Applicative
 import Prelude
 
-import Development.Shake.CmdOption
-import Development.Shake.Core
+import Development.Shake.Internal.CmdOption
+import Development.Shake.Internal.Core.Run
 import Development.Shake.FilePath
-import Development.Shake.FilePattern
-import Development.Shake.Types
-import Development.Shake.Rules.File
-import Development.Shake.Derived
+import Development.Shake.Internal.FilePattern
+import Development.Shake.Internal.Options
+import Development.Shake.Internal.Rules.File
+import Development.Shake.Internal.Derived
 
 ---------------------------------------------------------------------
 -- ACTUAL EXECUTION
@@ -95,11 +104,11 @@ data Result
     | ResultCode ExitCode
     | ResultTime Double
     | ResultLine String
-    | ResultProcess Pid
+    | ResultProcess PID
       deriving Eq
 
-data Pid = Pid0 | Pid ProcessHandle
-instance Eq Pid where _ == _ = True
+data PID = PID0 | PID ProcessHandle
+instance Eq PID where _ == _ = True
 
 
 ---------------------------------------------------------------------
@@ -121,14 +130,18 @@ commandExplicit funcName oopts results exe args = do
 
     let verboser act = do
             let cwd = listToMaybe $ reverse [x | Cwd x <- opts]
-            putLoud $ maybe "" (\x -> "cd " ++ x ++ "; ") cwd ++ showCommandForUser2 exe args
+            putLoud $
+                maybe "" (\x -> "cd " ++ x ++ "; ") cwd ++
+                if useShell then unwords $ exe : args else showCommandForUser2 exe args
             verb <- getVerbosity
+            -- run quietly to supress the tracer (don't want to print twice)
             (if verb >= Loud then quietly else id) act
 
     let tracer = case reverse [x | Traced x <- opts] of
             "":_ -> liftIO
             msg:_ -> traced msg
-            [] -> traced (takeFileName exe)
+            _ | useShell -> traced $ takeFileName $ fst $ word1 exe
+            [] -> traced $ takeFileName exe
 
     let tracker act
             | useLint = fsatrace act
@@ -137,7 +150,7 @@ commandExplicit funcName oopts results exe args = do
             | otherwise = act exe args
 
         shelled = runShell (unwords $ exe : args)
-                              
+
         ignore = map (?==) shakeLintIgnore
         ham cwd xs = [makeRelative cwd x | x <- map toStandard xs
                                          , any (`isPrefixOf` x) shakeLintInside
@@ -162,7 +175,7 @@ commandExplicit funcName oopts results exe args = do
                                     tmpdir <- getTemporaryDirectory
                                     let fake = tmpdir ++ "fsatrace-fakes" ++ re
                                     unlessM (doesFileExist fake) $ do
-                                        createDirectoryIfMissing True $ takeDirectory fake
+                                        createDirectoryRecursive $ takeDirectory fake
                                         copyFile re fake
                                     return fake
                                 else return re
@@ -196,7 +209,7 @@ commandExplicit funcName oopts results exe args = do
             pxs <- liftIO $ parseFSAT <$> readFileUTF8' file
             xs <- liftIO $ filterM doesFileExist [x | FSATRead x <- pxs]
             cwd <- liftIO getCurrentDirectory
-            unsafeAllowApply $ needNorm $ ham cwd xs
+            unsafeAllowApply $ need $ ham cwd xs
             return res
 
     skipper $ tracker $ \exe args -> verboser $ tracer $ commandExplicitIO funcName opts results exe args
@@ -248,7 +261,7 @@ commandExplicitIO funcName opts results exe args = do
             _ -> Nothing
     let optShell = Shell `elem` opts
     let optBinary = BinaryPipes `elem` opts
-    let optAsync = ResultProcess Pid0 `elem` results
+    let optAsync = ResultProcess PID0 `elem` results
     let optTimeout = listToMaybe $ reverse [x | Timeout x <- opts]
     let optWithStdout = last $ False : [x | WithStdout x <- opts]
     let optWithStderr = last $ True : [x | WithStderr x <- opts]
@@ -269,7 +282,7 @@ commandExplicitIO funcName opts results exe args = do
             ResultCode _ -> return ([], [], \_ _ ex -> return $ ResultCode ex)
             ResultTime _ -> return ([], [], \dur _ _ -> return $ ResultTime dur)
             ResultLine _ -> return ([], [], \_ _ _ -> return $ ResultLine cmdline)
-            ResultProcess _ -> return ([], [], \_ pid _ -> return $ ResultProcess $ Pid pid)
+            ResultProcess _ -> return ([], [], \_ pid _ -> return $ ResultProcess $ PID pid)
             ResultStdout    s -> do (a,b) <- buf s; return (a , [], \_ _ _ -> fmap ResultStdout b)
             ResultStderr    s -> do (a,b) <- buf s; return ([], a , \_ _ _ -> fmap ResultStderr b)
             ResultStdouterr s -> do (a,b) <- buf s; return (a , a , \_ _ _ -> fmap ResultStdouterr b)
@@ -283,29 +296,25 @@ commandExplicitIO funcName opts results exe args = do
         ,poStderr = [DestEcho | optEchoStderr] ++ map DestFile optFileStderr ++ [DestString exceptionBuffer | optWithStderr && not optAsync] ++ concat dStderr
         ,poAsync = optAsync
         }
-    res <- try_ $ duration $ process po
-
-    let failure extra = do
-            cwd <- case optCwd of
-                Nothing -> return ""
-                Just v -> do
-                    v <- canonicalizePath v `catch_` const (return v)
-                    return $ "Current directory: " ++ v ++ "\n"
-            fail $
-                "Development.Shake." ++ funcName ++ ", system command failed\n" ++
-                "Command: " ++ cmdline ++ "\n" ++
-                cwd ++ extra
-    case res of
-        Left err -> failure $ show err
-        Right (_,(_,ex)) | ex /= ExitSuccess && ResultCode ExitSuccess `notElem` results -> do
-            exceptionBuffer <- readBuffer exceptionBuffer
-            let captured = ["Stderr" | optWithStderr] ++ ["Stdout" | optWithStdout]
-            failure $
-                "Exit code: " ++ show (case ex of ExitFailure i -> i; _ -> 0) ++ "\n" ++
-                if null captured then "Stderr not captured because WithStderr False was used\n"
-                else if null exceptionBuffer then intercalate " and " captured ++ " " ++ (if length captured == 1 then "was" else "were") ++ " empty"
-                else intercalate " and " captured ++ ":\n" ++ unlines (dropWhile null $ lines $ concat exceptionBuffer)
-        Right (dur,(pid,ex)) -> mapM (\f -> f dur pid ex) resultBuild
+    (dur,(pid,exit)) <- duration $ process po
+    if exit == ExitSuccess || ResultCode ExitSuccess `elem` results then
+        mapM (\f -> f dur pid exit) resultBuild
+     else do
+        exceptionBuffer <- readBuffer exceptionBuffer
+        let captured = ["Stderr" | optWithStderr] ++ ["Stdout" | optWithStdout]
+        cwd <- case optCwd of
+            Nothing -> return ""
+            Just v -> do
+                v <- canonicalizePath v `catchIO` const (return v)
+                return $ "Current directory: " ++ v ++ "\n"
+        fail $
+            "Development.Shake." ++ funcName ++ ", system command failed\n" ++
+            "Command: " ++ cmdline ++ "\n" ++
+            cwd ++
+            "Exit code: " ++ show (case exit of ExitFailure i -> i; _ -> 0) ++ "\n" ++
+            if null captured then "Stderr not captured because WithStderr False was used\n"
+            else if null exceptionBuffer then intercalate " and " captured ++ " " ++ (if length captured == 1 then "was" else "were") ++ " empty"
+            else intercalate " and " captured ++ ":\n" ++ unlines (dropWhile null $ lines $ concat exceptionBuffer)
 
 
 -- | Apply all environment operations, to produce a new environment to use.
@@ -387,8 +396,8 @@ newtype Stdouterr a = Stdouterr {fromStdouterr :: a}
 newtype Exit = Exit {fromExit :: ExitCode}
 
 -- | Collect the 'ProcessHandle' of the process.
---   If you do collect the process handle, the command will run asyncronously and the call to 'cmd'/'command'
---   will return as soon as the process is spawned. Any 'Stdout'\/'Stderr' captures will return empty strings.
+--   If you do collect the process handle, the command will run asyncronously and the call to 'cmd' \/ 'command'
+--   will return as soon as the process is spawned. Any 'Stdout' \/ 'Stderr' captures will return empty strings.
 newtype Process = Process {fromProcess :: ProcessHandle}
 
 -- | Collect the time taken to execute the process. Can be used in conjunction with 'CmdLine' to
@@ -417,6 +426,18 @@ instance CmdString String where cmdString = (Str "", \(Str x) -> x)
 instance CmdString BS.ByteString where cmdString = (BS BS.empty, \(BS x) -> x)
 instance CmdString LBS.ByteString where cmdString = (LBS LBS.empty, \(LBS x) -> x)
 
+
+#if __GLASGOW_HASKELL__ >= 710
+class Unit a
+instance {-# OVERLAPPING #-} Unit b => Unit (a -> b)
+instance {-# OVERLAPPABLE #-} a ~ () => Unit (m a)
+#else
+class Unit a
+instance Unit b => Unit (a -> b)
+instance a ~ () => Unit (m a)
+#endif
+
+
 -- | A class for specifying what results you want to collect from a process.
 --   Values are formed of 'Stdout', 'Stderr', 'Exit' and tuples of those.
 class CmdResult a where
@@ -431,10 +452,10 @@ instance CmdResult ExitCode where
     cmdResult = ([ResultCode ExitSuccess], \[ResultCode x] -> x)
 
 instance CmdResult Process where
-    cmdResult = ([ResultProcess Pid0], \[ResultProcess (Pid x)] -> Process x)
+    cmdResult = ([ResultProcess PID0], \[ResultProcess (PID x)] -> Process x)
 
 instance CmdResult ProcessHandle where
-    cmdResult = ([ResultProcess Pid0], \[ResultProcess (Pid x)] -> x)
+    cmdResult = ([ResultProcess PID0], \[ResultProcess (PID x)] -> x)
 
 instance CmdResult CmdLine where
     cmdResult = ([ResultLine ""], \[ResultLine x] -> CmdLine x)
@@ -528,18 +549,17 @@ type a :-> t = a
 --   As some examples, here are some calls, and the resulting command string:
 --
 -- @
--- 'unit' $ 'cmd' \"git log --pretty=\" \"oneline\"           -- git log --pretty= oneline
--- 'unit' $ 'cmd' \"git log --pretty=\" [\"oneline\"]         -- git log --pretty= oneline
--- 'unit' $ 'cmd' \"git log\" (\"--pretty=\" ++ \"oneline\")    -- git log --pretty=oneline
--- 'unit' $ 'cmd' \"git log\" (\"--pretty=\" ++ \"one line\")   -- git log --pretty=one line
--- 'unit' $ 'cmd' \"git log\" [\"--pretty=\" ++ \"one line\"]   -- git log "--pretty=one line"
+-- 'cmd_' \"git log --pretty=\" \"oneline\"           -- git log --pretty= oneline
+-- 'cmd_' \"git log --pretty=\" [\"oneline\"]         -- git log --pretty= oneline
+-- 'cmd_' \"git log\" (\"--pretty=\" ++ \"oneline\")    -- git log --pretty=oneline
+-- 'cmd_' \"git log\" (\"--pretty=\" ++ \"one line\")   -- git log --pretty=one line
+-- 'cmd_' \"git log\" [\"--pretty=\" ++ \"one line\"]   -- git log "--pretty=one line"
 -- @
 --
 --   More examples, including return values, see this translation of the examples given for the 'command' function:
 --
 -- @
--- () <- 'cmd' \"gcc -c myfile.c\"                                  -- compile a file, throwing an exception on failure
--- 'unit' $ 'cmd' \"gcc -c myfile.c\"                                 -- alternative to () <- binding.
+-- 'cmd_' \"gcc -c myfile.c\"                                         -- compile a file, throwing an exception on failure
 -- 'Exit' c <- 'cmd' \"gcc -c\" [myfile]                              -- run a command, recording the exit code
 -- ('Exit' c, 'Stderr' err) <- 'cmd' \"gcc -c myfile.c\"                -- run a command, recording the exit code and error output
 -- 'Stdout' out <- 'cmd' \"gcc -MM myfile.c\"                         -- run a command, recording the output
@@ -549,8 +569,7 @@ type a :-> t = a
 --   When passing file arguments we use @[myfile]@ so that if the @myfile@ variable contains spaces they are properly escaped.
 --
 --   If you use 'cmd' inside a @do@ block and do not use the result, you may get a compile-time error about being
---   unable to deduce 'CmdResult'. To avoid this error, bind the result to @()@, or include a type signature, or use
---   the 'unit' function.
+--   unable to deduce 'CmdResult'. To avoid this error, use 'cmd_'.
 --
 --   The 'cmd' function can also be run in the 'IO' monad, but then 'Traced' is ignored and command lines are not echoed.
 --   As an example:
@@ -559,31 +578,43 @@ type a :-> t = a
 -- 'cmd' ('Cwd' \"generated\") 'Shell' \"gcc -c myfile.c\" :: IO ()
 -- @
 cmd :: CmdArguments args => args :-> Action r
-cmd = cmdArguments []
+cmd = cmdArguments mempty
+
+-- | See 'cmd'. Same as 'cmd' except with a unit result.
+-- 'cmd' is to 'cmd_' as 'command' is to 'command_'.
+cmd_ :: (CmdArguments args, Unit args) => args :-> Action ()
+cmd_ = cmd
 
 -- | The arguments to 'cmd' - see 'cmd' for examples and semantics.
-class CmdArguments t where cmdArguments :: [Either CmdOption String] -> t
-instance (Arg a, CmdArguments r) => CmdArguments (a -> r) where
-    cmdArguments xs x = cmdArguments $ xs ++ arg x
+newtype CmdArgument = CmdArgument [Either CmdOption String]
+  deriving (Eq, Semigroup, Monoid, Show)
+
+-- | The arguments to 'cmd' - see 'cmd' for examples and semantics.
+class CmdArguments t where
+    -- | Arguments to cmd
+    cmdArguments :: CmdArgument -> t
+instance (IsCmdArgument a, CmdArguments r) => CmdArguments (a -> r) where
+    cmdArguments xs x = cmdArguments $ xs `mappend` toCmdArgument x
 instance CmdResult r => CmdArguments (Action r) where
-    cmdArguments x = case partitionEithers x of
+    cmdArguments (CmdArgument x) = case partitionEithers x of
         (opts, x:xs) -> let (a,b) = cmdResult in b <$> commandExplicit "cmd" opts a x xs
         _ -> error "Error, no executable or arguments given to Development.Shake.cmd"
 instance CmdResult r => CmdArguments (IO r) where
-    cmdArguments x = case partitionEithers x of
+    cmdArguments (CmdArgument x) = case partitionEithers x of
         (opts, x:xs) -> let (a,b) = cmdResult in b <$> commandExplicitIO "cmd" opts a x xs
         _ -> error "Error, no executable or arguments given to Development.Shake.cmd"
-
-instance CmdArguments [Either CmdOption String] where
+instance CmdArguments CmdArgument where
     cmdArguments = id
 
-
-class Arg a where arg :: a -> [Either CmdOption String]
-instance Arg String where arg = map Right . words
-instance Arg [String] where arg = map Right
-instance Arg CmdOption where arg = return . Left
-instance Arg [CmdOption] where arg = map Left
-instance Arg a => Arg (Maybe a) where arg = maybe [] arg
+-- | Class to convert an a  to a CmdArgument
+class IsCmdArgument a where
+    -- | Conversion to a CmdArgument
+    toCmdArgument :: a -> CmdArgument
+instance IsCmdArgument String where toCmdArgument = CmdArgument . map Right . words
+instance IsCmdArgument [String] where toCmdArgument = CmdArgument . map Right
+instance IsCmdArgument CmdOption where toCmdArgument = CmdArgument . return . Left
+instance IsCmdArgument [CmdOption] where toCmdArgument = CmdArgument . map Left
+instance IsCmdArgument a => IsCmdArgument (Maybe a) where toCmdArgument = maybe mempty toCmdArgument
 
 
 ---------------------------------------------------------------------
