@@ -1,12 +1,13 @@
-{-# LANGUAGE GeneralizedNewtypeDeriving, FlexibleInstances, MultiParamTypeClasses #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
 
 module General.Intern(
     Intern, Id(..),
     empty, insert, add, lookup, toList, fromList
     ) where
 
-import General.Binary
 import Development.Shake.Classes
+import Foreign.Storable
+import Data.Word
 import Prelude hiding (lookup)
 import qualified Data.HashMap.Strict as Map
 import Data.List(foldl')
@@ -16,12 +17,7 @@ import Data.List(foldl')
 data Intern a = Intern {-# UNPACK #-} !Word32 !(Map.HashMap a Id)
 
 newtype Id = Id Word32
-    deriving (Eq,Hashable,Binary,Show,NFData)
-
-instance BinaryWith w Id where
-    putWith ctx = put
-    getWith ctx = get
-
+    deriving (Eq,Hashable,Binary,Show,NFData,Storable)
 
 empty :: Intern a
 empty = Intern 0 Map.empty
