@@ -21,8 +21,8 @@ import System.Process
 import System.Time.Extra
 import Data.Unique
 import Data.IORef
-import qualified Data.ByteString.Char8 as BS
 import qualified Data.ByteString.Internal as BS(createAndTrim)
+import qualified Data.ByteString.Char8 as BS
 import qualified Data.ByteString.Lazy as LBS
 import General.Extra
 import Prelude
@@ -92,17 +92,18 @@ stdStream file _ _ = CreatePipe
 stdIn :: (FilePath -> Handle) -> [Source] -> (StdStream, Handle -> IO ())
 stdIn file [] = (Inherit, const $ return ())
 stdIn file [SrcFile x] = (UseHandle $ file x, const $ return ())
-stdIn file src = (,) CreatePipe $ \h ->
-    void $ tryBool isPipeGone $ do
-        forM_ src $ \x -> case x of
-            SrcString x -> hPutStr h x
-            SrcBytes x -> LBS.hPutStr h x
-            SrcFile x -> LBS.hPutStr h =<< LBS.hGetContents (file x)
-        hFlush h
-        hClose h
-    where
-        isPipeGone IOError{ioe_type=ResourceVanished, ioe_errno=Just ioe} = Errno ioe == ePIPE
-        isPipeGone _ = False
+stdIn file src = (,) CreatePipe $ \h -> ignoreSigPipe $ do
+    forM_ src $ \x -> case x of
+        SrcString x -> hPutStr h x
+        SrcBytes x -> LBS.hPutStr h x
+        SrcFile x -> LBS.hPutStr h =<< LBS.hGetContents (file x)
+    hClose h
+
+
+ignoreSigPipe :: IO () -> IO ()
+ignoreSigPipe = handleIO $ \e -> case e of
+    IOError {ioe_type=ResourceVanished, ioe_errno=Just ioe} | Errno ioe == ePIPE -> return ()
+    _ -> throwIO e
 
 
 withTimeout :: Maybe Double -> IO () -> IO a -> IO a
@@ -130,19 +131,6 @@ abort pid = do
     -- seems to happen with some GHC 7.2 compiled binaries with FFI etc
     terminateProcess pid
 
--- FIXME: There is a new withCreateProcess in process-1.4.3.0 which is probably better than ours...
-withCreateProcessOld :: CreateProcess -> ((Maybe Handle, Maybe Handle, Maybe Handle, ProcessHandle) -> IO a) -> IO a
-withCreateProcessOld cp act = mask $ \restore -> do
-    ans@(inh, outh, errh, pid) <- createProcess cp
-    onException (restore $ act ans) $ do
-        mapM_ (`whenJust` hClose) [inh, outh, errh]
-        ignore $ do
-            -- sometimes we fail before the process is valid
-            -- therefore if terminate process fails, skip waiting on the process
-            terminateProcess pid
-            void $ waitForProcess pid
-
-
 withFiles :: IOMode -> [FilePath] -> ((FilePath -> Handle) -> IO a) -> IO a
 withFiles mode files act = withs (map (`withFile` mode) files) $ \handles ->
     act $ \x -> fromJust $ lookup x $ zip files handles
@@ -158,7 +146,7 @@ process po = do
         let cp = (cmdSpec poCommand){cwd = poCwd, env = poEnv, create_group = isJust poTimeout, close_fds = True
                  ,std_in = fst $ stdIn inHandle poStdin
                  ,std_out = stdStream outHandle poStdout poStderr, std_err = stdStream outHandle poStderr poStdout}
-        withCreateProcessOld cp $ \(inh, outh, errh, pid) ->
+        withCreateProcessCompat cp $ \inh outh errh pid ->
             withTimeout poTimeout (abort pid) $ do
 
                 let streams = [(outh, stdout, poStdout) | Just outh <- [outh], CreatePipe <- [std_out cp]] ++
@@ -182,7 +170,7 @@ process po = do
                             DestString x -> addBuffer x . (if isWindows then replace "\r\n" "\n" else id) . BS.unpack
                             DestBytes x -> addBuffer x
                         forkWait $ whileM $ do
-                            src <- bs_hGetSome h 4096
+                            src <- bsHGetSome h 4096
                             mapM_ ($ src) dest
                             notM $ hIsEOF h
                      else if isTied then do
@@ -219,7 +207,20 @@ process po = do
 ---------------------------------------------------------------------
 -- COMPATIBILITY
 
--- available in bytestring-0.9.1.10 and above
+-- available in bytestring-0.9.1.10, GHC 7.8 and above
 -- implementation copied below
-bs_hGetSome :: Handle -> Int -> IO BS.ByteString
-bs_hGetSome h i = BS.createAndTrim i $ \p -> hGetBufSome h p i
+bsHGetSome :: Handle -> Int -> IO BS.ByteString
+bsHGetSome h i = BS.createAndTrim i $ \p -> hGetBufSome h p i
+
+-- available in process-1.4.3.0, GHC ??? (Nov 2015)
+-- logic copied directly (apart from Ctrl-C handling magic using internal pieces)
+withCreateProcessCompat :: CreateProcess -> (Maybe Handle -> Maybe Handle -> Maybe Handle -> ProcessHandle -> IO a) -> IO a
+withCreateProcessCompat cp act = bracketOnError (createProcess cp) cleanup
+    (\(m_in, m_out, m_err, ph) -> act m_in m_out m_err ph)
+    where
+        cleanup (inh, outh, errh, pid) = do
+            terminateProcess pid
+            whenJust inh $ ignoreSigPipe . hClose
+            whenJust outh hClose
+            whenJust errh hClose
+            forkIO $ void $ waitForProcess pid
