@@ -4,7 +4,6 @@ module General.Template(runTemplate) where
 
 import System.FilePath.Posix
 import Control.Exception.Extra
-import Control.Monad
 import Control.Monad.IO.Class
 import Data.Char
 import qualified Data.ByteString.Lazy.Char8 as LBS
@@ -23,14 +22,15 @@ libraries =
 -- * <script src="foo"></script> ==> <script>[[foo]]</script>
 --
 -- * <link href="foo" rel="stylesheet" type="text/css" /> ==> <style type="text/css">[[foo]]</style>
-runTemplate :: MonadIO m => (FilePath -> m LBS.ByteString) -> LBS.ByteString -> m LBS.ByteString
-runTemplate ask = liftM LBS.unlines . mapM f . LBS.lines
+runTemplate :: (Functor m, MonadIO m) => (FilePath -> m LBS.ByteString) -> LBS.ByteString -> m LBS.ByteString
+-- Functor constraint is required for GHC 7.8 and before
+runTemplate ask = fmap LBS.unlines . mapM f . LBS.lines
     where
         link = LBS.pack "<link href=\""
         script = LBS.pack "<script src=\""
 
-        f x | Just file <- lbs_stripPrefix script y = do res <- grab file; return $ LBS.pack "<script>\n" `LBS.append` res `LBS.append` LBS.pack "\n</script>"
-            | Just file <- lbs_stripPrefix link y = do res <- grab file; return $ LBS.pack "<style type=\"text/css\">\n" `LBS.append` res `LBS.append` LBS.pack "\n</style>"
+        f x | Just file <- lbsStripPrefix script y = do res <- grab file; return $ LBS.pack "<script>\n" `LBS.append` res `LBS.append` LBS.pack "\n</script>"
+            | Just file <- lbsStripPrefix link y = do res <- grab file; return $ LBS.pack "<style type=\"text/css\">\n" `LBS.append` res `LBS.append` LBS.pack "\n</style>"
             | otherwise = return x
             where
                 y = LBS.dropWhile isSpace x
@@ -42,6 +42,11 @@ runTemplate ask = liftM LBS.unlines . mapM f . LBS.lines
         asker x = ask x
 
 
-lbs_stripPrefix :: LBS.ByteString -> LBS.ByteString -> Maybe LBS.ByteString
-lbs_stripPrefix prefix text = if a == prefix then Just b else Nothing
+---------------------------------------------------------------------
+-- COMPATIBILITY
+
+-- available in bytestring-0.10.8.0, GHC 8.0 and above
+-- alternative implementation below
+lbsStripPrefix :: LBS.ByteString -> LBS.ByteString -> Maybe LBS.ByteString
+lbsStripPrefix prefix text = if a == prefix then Just b else Nothing
     where (a,b) = LBS.splitAt (LBS.length prefix) text
