@@ -21,11 +21,11 @@ import qualified Data.ByteString.Lazy.Char8 as LBS
 import Prelude
 
 
-main = shakenCwd test $ \args obj -> do
+main = shakeTest_ test $ do
     -- shake_helper must be in a subdirectory so we can test placing that subdir on the $PATH
-    let helper = toNative $ obj "helper/shake_helper" <.> exe
-    let name !> test = do want [name | null args || name `elem` args]
-                          name ~> do need [obj "helper/shake_helper" <.> exe]; test
+    let helper = toNative $ "helper/shake_helper" <.> exe
+    let name !> test = do want [name]
+                          name ~> do need ["helper/shake_helper" <.> exe]; test
 
     let helper_source = unlines
             ["import Control.Concurrent"
@@ -51,12 +51,12 @@ main = shakenCwd test $ \args obj -> do
             ,"        hFlush stderr"
             ]
 
-    obj "shake_helper.hs" %> \out -> do
+    "shake_helper.hs" %> \out -> do
         need ["../../src/Test/Command.hs"]
         writeFileChanged out helper_source
-    [obj "helper/shake_helper" <.> exe, obj "shake_helper.o", obj "shake_helper.hi"] &%> \_ -> do
-        need [obj "shake_helper.hs"]
-        cmd (Cwd $ obj "") "ghc --make" "shake_helper.hs -o helper/shake_helper"
+    ["helper/shake_helper" <.> exe, "shake_helper.o", "shake_helper.hi"] &%> \_ -> do
+        need ["shake_helper.hs"]
+        cmd "ghc --make" "shake_helper.hs -o helper/shake_helper"
 
     "capture" !> do
         (Stderr err, Stdout out) <- cmd helper ["ostuff goes here","eother stuff here"]
@@ -74,19 +74,25 @@ main = shakenCwd test $ \args obj -> do
 
     "cwd" !> do
         -- FIXME: Linux searches the Cwd argument for the file, Windows searches getCurrentDirectory
-        helper <- liftIO $ canonicalizePath $ obj "helper/shake_helper" <.> exe
-        Stdout out <- cmd (Cwd $ obj "") helper "c"
+        helper <- liftIO $ canonicalizePath $ "helper/shake_helper" <.> exe
+        Stdout out <- cmd (Cwd "helper") helper "c"
         let norm = fmap dropTrailingPathSeparator . canonicalizePath . trim
-        liftIO $ join $ liftM2 (===) (norm out) (norm $ obj "")
+        liftIO $ join $ liftM2 (===) (norm out) (norm "helper")
 
-    "timeout" !> do
-        opts <- getShakeOptions
-        offset <- liftIO offsetTime
+    let checkTimeout act = do
+            offset <- liftIO offsetTime
+            act
+            t <- liftIO offset
+            putNormal $ "Timed out in " ++ showDuration t
+            when (t < 2 || t > 8) $ error $ "failed to timeout, took " ++ show t
+
+    "timeout1" !> checkTimeout (do
         Exit exit <- cmd (Timeout 2) helper "w20"
-        t <- liftIO offset
-        putNormal $ "Timed out in " ++ showDuration t
-        when (exit == ExitSuccess) $ error "== ExitSuccess"
-        when (t < 2 || t > 8) $ error $ "failed to timeout, took " ++ show t
+        liftIO $ assertBool (exit /= ExitSuccess) "exit was ExitSuccess")
+
+    "timeout2" !> do checkTimeout $ liftIO $ timeout 2 $ cmd_ helper "w20"
+
+    when False $ "timeout3" !> do checkTimeout $ liftIO $ timeout 2 $ cmd_ Shell helper "w20"
 
     "env" !> do
         -- use liftIO since it blows away PATH which makes lint-tracker stop working
@@ -108,15 +114,15 @@ main = shakenCwd test $ \args obj -> do
             fail $ "Invalid CmdLine, " ++ x
 
     "path" !> do
-        let path = AddPath [dropTrailingPathSeparator $ obj "helper"] []
-        unit $ cmd $ obj "helper/shake_helper"
-        unit $ cmd $ obj "helper/shake_helper" <.> exe
-        unit $ cmd path Shell "shake_helper"
-        unit $ cmd path "shake_helper"
+        let path = AddPath [dropTrailingPathSeparator "helper"] []
+        cmd_ "helper/shake_helper"
+        cmd_ $ "helper/shake_helper" <.> exe
+        cmd_ path Shell "shake_helper"
+        cmd_ path "shake_helper"
 
     "file" !> do
-        let file = obj "file.txt"
-        unit $ cmd helper (FileStdout file) (FileStderr file) (EchoStdout False) (EchoStderr False) (WithStderr False) "ofoo ebar obaz"
+        let file = "file.txt"
+        cmd_ helper (FileStdout file) (FileStderr file) (EchoStdout False) (EchoStderr False) (WithStderr False) "ofoo ebar obaz"
         liftIO $ assertContents file "foo\nbar\nbaz\n"
         liftIO $ waits $ \w -> do
             Stderr err <- cmd helper (FileStdout file) (FileStderr file) ["ofoo",w,"ebar",w,"obaz"]
@@ -147,14 +153,14 @@ main = shakenCwd test $ \args obj -> do
             liftIO $ x === "hello world"
 
     "async" !> do
-        let file = obj "async.txt"
+        let file = "async.txt"
         pid <- cmd helper (FileStdout file) "w2" "ohello"
         Nothing <- liftIO $ getProcessExitCode pid
         ExitSuccess <- liftIO $ waitForProcess pid
         liftIO $ assertContents file "hello\n"
 
 
-test build obj = do
+test build = do
     -- reduce the overhead by running all the tests in parallel
     -- lint can make a big different to the command lines, so test with and without
     whenM hasTracker $
