@@ -7,7 +7,8 @@ import Test.Type
 import Data.List
 import Data.Function
 import Control.Monad
-import System.Directory(getCurrentDirectory, setCurrentDirectory, createDirectory, createDirectoryIfMissing)
+import General.Extra
+import System.Directory(getCurrentDirectory, createDirectory)
 import qualified System.Directory as IO
 import qualified System.IO.Extra as IO
 
@@ -26,49 +27,45 @@ showEsc = concatMap f
           f x = [x]
 
 
-main = shakenCwd test $ \args obj -> do
-    let unobj = id
-    want $ map obj args
-    obj "*.contents" %> \out ->
-        writeFileLines out =<< getDirectoryContents (obj $ readEsc $ dropExtension $ unobj out)
-    obj "*.dirs" %> \out ->
-        writeFileLines out =<< getDirectoryDirs (obj $ readEsc $ dropExtension $ unobj out)
-    obj "*.files" %> \out -> do
-        let pats = readEsc $ dropExtension $ unobj out
+main = shakeTest_ test $ do
+    "*.contents" %> \out ->
+        writeFileLines out =<< getDirectoryContents (readEsc $ dropExtension out)
+    "*.dirs" %> \out ->
+        writeFileLines out =<< getDirectoryDirs (readEsc $ dropExtension out)
+    "*.files" %> \out -> do
+        let pats = readEsc $ dropExtension out
         let (x:xs) = ["" | " " `isPrefixOf` pats] ++ words pats
-        writeFileLines out . map toStandard =<< getDirectoryFiles (obj x) xs
+        writeFileLines out . map toStandard =<< getDirectoryFiles x xs
 
-    obj "*.exist" %> \out -> do
-        let xs = map obj $ words $ readEsc $ dropExtension $ unobj out
+    "*.exist" %> \out -> do
+        let xs = words $ readEsc $ dropExtension out
         fs <- mapM doesFileExist xs
         ds <- mapM doesDirectoryExist xs
         let bool x = if x then "1" else "0"
         writeFileLines out $ zipWith ((++) `on` bool) fs ds
 
-    obj "dots" %> \out -> do
+    "dots" %> \out -> do
         cwd <- liftIO getCurrentDirectory
-        liftIO $ setCurrentDirectory $ obj ""
         b1 <- liftM2 (==) (getDirectoryContents ".") (getDirectoryContents "")
         b2 <- liftM2 (==) (getDirectoryDirs ".") (getDirectoryDirs "")
         b3 <- liftM2 (==) (getDirectoryFiles "." ["*.txt"]) (getDirectoryFiles "" ["*.txt"])
         b4 <- liftM2 (==) (getDirectoryFiles "." ["C.txt/*.txt"]) (getDirectoryFiles "" ["C.txt/*.txt"])
         b5 <- liftM2 (==) (getDirectoryFiles "." ["//*.txt"]) (getDirectoryFiles "" ["//*.txt"])
-        liftIO $ setCurrentDirectory cwd
         writeFileLines out $ map show [b1,b2,b3,b4,b5]
 
-test build obj = do
-    let demand x ys = let f = showEsc x in do build [f]; assertContents (obj f) $ unlines $ words ys
+test build = do
+    let demand x ys = let f = showEsc x in do build [f]; assertContents f $ unlines $ words ys
     build ["clean"]
     demand " *.txt.files" ""
     demand " //*.txt.files" ""
     demand ".dirs" ""
     demand "A.txt B.txt C.txt.exist" "00 00 00"
 
-    writeFile (obj "A.txt") ""
-    writeFile (obj "B.txt") ""
-    createDirectory (obj "C.txt")
-    writeFile (obj "C.txt/D.txt") ""
-    writeFile (obj "C.txt/E.xtx") ""
+    writeFile "A.txt" ""
+    writeFile "B.txt" ""
+    createDirectory "C.txt"
+    writeFile "C.txt/D.txt" ""
+    writeFile "C.txt/E.xtx" ""
     demand " *.txt.files" "A.txt B.txt"
     demand ".dirs" "C.txt"
     demand "A.txt B.txt C.txt.exist" "10 10 01"
@@ -83,16 +80,16 @@ test build obj = do
     assertException ["missing_dir","does not exist"] $ build ["--quiet",showEsc "missing_dir *.files"]
 
     build ["dots","--no-lint"]
-    assertContents (obj "dots") $ unlines $ words "True True True True True"
+    assertContents "dots" $ unlines $ words "True True True True True"
 
     let removeTest pat del keep =
             IO.withTempDir $ \dir -> do
                 forM_ (del ++ keep) $ \s -> do
-                    createDirectoryIfMissing True $ dir </> takeDirectory s
+                    createDirectoryRecursive $ dir </> takeDirectory s
                     unless (hasTrailingPathSeparator s) $
                         writeFile (dir </> s) ""
                 removeFiles dir pat
-                createDirectoryIfMissing True dir
+                createDirectoryRecursive dir
                 forM_ (map ((,) False) del ++ map ((,) True) keep) $ \(b,s) -> do
                     b2 <- (if hasTrailingPathSeparator s then IO.doesDirectoryExist else IO.doesFileExist) $ dir </> s
                     when (b /= b2) $ do
