@@ -4,66 +4,75 @@ module Test.Docs(main) where
 
 import Development.Shake
 import Development.Shake.FilePath
+import System.Directory
 import Test.Type
 import Control.Monad
 import Data.Char
+import General.Extra
 import Data.List.Extra
 import Data.Maybe
 import System.Info
 import Data.Version.Extra
 
 
--- Older versions of Haddock garbage the --@ markup
-brokenHaddock = compilerVersion < makeVersion [7,8]
+-- Older versions of Haddock garbage the --@ markup and have ambiguity errors
+brokenHaddock = compilerVersion < makeVersion [8]
 
-main = shaken (\a b -> unless brokenHaddock $ noTest a b) $ \args obj -> do
-    let index = obj "dist/doc/html/shake/index.html"
-    let config = obj "dist/setup-config"
-    want [obj "Success.txt"]
+main = shakeTest_ (unless brokenHaddock . noTest) $ do
+    let index = "dist/doc/html/shake/index.html"
+    let config = "dist/setup-config"
+    want ["Success.txt"]
 
-    want $ map (\x -> fromMaybe (obj x) $ stripPrefix "!" x) args
-
-    let needSource = need =<< getDirectoryFiles "." ["src/Development/Shake.hs","src/Development/Shake//*.hs","src/Development/Ninja/*.hs","src/General//*.hs"]
+    let needSource = need =<< getDirectoryFiles "." (map (root </>)
+            ["src/Development/Shake.hs","src/Development/Shake//*.hs","src/Development/Ninja/*.hs","src/General//*.hs"])
 
     config %> \_ -> do
-        need ["shake.cabal","Setup.hs"]
+        need $ map (root </>) ["shake.cabal","Setup.hs"]
         -- Make Cabal and Stack play nicely
         path <- getEnv "GHC_PACKAGE_PATH"
-        unit $ cmd (RemEnv "GHC_PACKAGE_PATH") "runhaskell Setup.hs configure"
-            ["--builddir=" ++ obj "dist","--user"]
+        liftIO $ createDirectoryRecursive "dist"
+        dist <- liftIO $ canonicalizePath "dist" -- make sure it works even if we cwd
+        cmd_ (RemEnv "GHC_PACKAGE_PATH") (Cwd root) "runhaskell Setup.hs configure"
+            ["--builddir=" ++ dist,"--user"]
             -- package-db is very sensitive, see #267
-            ["--package-db=" ++ x | x <- maybe [] (filter (`notElem` [".",""]) . splitSearchPath) path]
-        trackAllow [obj "dist//*"]
+            -- note that the reverse ensures the behaviour is consistent between the flags and the env variable
+            ["--package-db=" ++ x | x <- maybe [] (reverse . filter (`notElem` [".",""]) . splitSearchPath) path]
+        -- Paths_shake is only created by "Setup build" (which we want to skip), and required by "Setup haddock", so we fake it
+        copyFile' (root </> "src/Paths.hs") "dist/build/autogen/Paths_shake.hs"
+        copyFile' (root </> "src/Paths.hs") "dist/build/shake/autogen/Paths_shake.hs"
+        writeFile' "dist/build/autogen/cabal_macros.h" ""
+        writeFile' "dist/build/shake/autogen/cabal_macros.h" ""
+        trackAllow ["dist//*"]
 
     index %> \_ -> do
-        need [config,"shake.cabal","Setup.hs","README.md","CHANGES.txt"]
+        need $ config : map (root </>) ["shake.cabal","Setup.hs","README.md","CHANGES.txt"]
         needSource
-        trackAllow [obj "dist//*"]
-        cmd "runhaskell Setup.hs haddock" ["--builddir=" ++ obj "dist"]
+        trackAllow ["dist//*"]
+        dist <- liftIO $ canonicalizePath "dist"
+        cmd (Cwd root) "runhaskell Setup.hs haddock" ["--builddir=" ++ dist]
 
-    obj "Paths_shake.hs" %> \out ->
-        copyFile' "src/Paths.hs" out
-
-    obj "Part_*.hs" %> \out -> do
-        need ["src/Test/Docs.hs"] -- so much of the generator is in this module
+    "Part_*.hs" %> \out -> do
+        need [root </> "src/Test/Docs.hs"] -- so much of the generator is in this module
         let noR = filter (/= '\r')
         src <- if "_md" `isSuffixOf` takeBaseName out then
-            fmap (findCodeMarkdown . lines . noR) $ readFile' $ "docs/" ++ drop 5 (reverse (drop 3 $ reverse $ takeBaseName out)) ++ ".md"
+            fmap (findCodeMarkdown . lines . noR) $ readFile' $ root </> "docs/" ++ drop 5 (reverse (drop 3 $ reverse $ takeBaseName out)) ++ ".md"
          else
-            fmap (findCodeHaddock . noR) $ readFile' $ obj $ "dist/doc/html/shake/" ++ replace "_" "-" (drop 5 $ takeBaseName out) ++ ".html"
+            fmap (findCodeHaddock . noR) $ readFile' $ "dist/doc/html/shake/" ++ replace "_" "-" (drop 5 $ takeBaseName out) ++ ".html"
 
         let (imports,rest) = partition ("import " `isPrefixOf`) $ showCode src
         writeFileChanged out $ unlines $
-            ["{-# LANGUAGE DeriveDataTypeable, RankNTypes, MultiParamTypeClasses, ExtendedDefaultRules, GeneralizedNewtypeDeriving #-}"
-            ,"{-# LANGUAGE NoMonomorphismRestriction, ScopedTypeVariables, ConstraintKinds #-}"
+            ["{-# LANGUAGE DeriveDataTypeable, RankNTypes, ExtendedDefaultRules, GeneralizedNewtypeDeriving #-}"
+            ,"{-# LANGUAGE NoMonomorphismRestriction, ScopedTypeVariables, ConstraintKinds, FlexibleContexts, TypeFamilies #-}"
             ,"{-# OPTIONS_GHC -w #-}"
             ,"module " ++ takeBaseName out ++ "() where"
             ,"import Control.Applicative"
             ,"import Control.Concurrent"
+            ,"import Control.Exception"
             ,"import Control.Monad"
             ,"import Data.ByteString(ByteString)"
             ,"import Data.Char"
             ,"import Data.Data"
+            ,"import Data.Dynamic"
             ,"import Data.List.Extra"
             ,"import System.Time.Extra"
             ,"import Data.Maybe"
@@ -76,6 +85,7 @@ main = shaken (\a b -> unless brokenHaddock $ noTest a b) $ \args obj -> do
             ,"import System.Console.GetOpt"
             ,"import System.Directory(setCurrentDirectory)"
             ,"import qualified System.Directory"
+            ,"import System.Environment(lookupEnv, getEnvironment)"
             ,"import System.Process"
             ,"import System.Exit"
             ,"import Control.Applicative"
@@ -111,32 +121,39 @@ main = shaken (\a b -> unless brokenHaddock $ noTest a b) $ \args obj -> do
             ,"(r1,r2) = (return () :: Rules(), return () :: Rules())"
             ,"xs = []"
             ,"ys = []"
+            ,"os = [\"file.o\"]"
             ,"out = \"\""
             ,"str1 = \"\""
             ,"str2 = \"\""
+            ,"def = undefined"
+            ,"var = undefined"
             ,"str = \"\""] ++
             rest
 
-    obj "Files.lst" %> \out -> do
-        need ["src/Test/Docs.hs"] -- so much of the generator is in this module
-        need [index,obj "Paths_shake.hs"]
-        filesHs <- getDirectoryFiles (obj "dist/doc/html/shake") ["Development-*.html"]
-        filesMd <- getDirectoryFiles "docs" ["*.md"]
+    "Files.lst" %> \out -> do
+        need [root </> "src/Test/Docs.hs"] -- so much of the generator is in this module
+        need [index]
+        filesHs <- getDirectoryFiles "dist/doc/html/shake" ["Development-*.html"]
+        filesMd <- getDirectoryFiles (root </> "docs") ["*.md"]
         writeFileChanged out $ unlines $
-            ["Part_" ++ replace "-" "_" (takeBaseName x) | x <- filesHs, not $ "-Classes.html" `isSuffixOf` x] ++
-            ["Part_" ++ takeBaseName x ++ "_md" | x <- filesMd, takeBaseName x `notElem` ["Developing","Model"]]
+            ["Part_" ++ replace "-" "_" (takeBaseName x) | x <- filesHs,
+                not $ any (`isSuffixOf` x) ["-Classes.html", "-FilePath.html"]] ++
+            ["Part_" ++ takeBaseName x ++ "_md" | x <- filesMd,
+                takeBaseName x `notElem` ["Developing","Model"]]
 
-    let needModules = do mods <- readFileLines $ obj "Files.lst"; need [obj m <.> "hs" | m <- mods]; return mods
+    let needModules = do mods <- readFileLines "Files.lst"; need [m <.> "hs" | m <- mods]; return mods
 
-    obj "Main.hs" %> \out -> do
+    "Main.hs" %> \out -> do
         mods <- needModules
         writeFileLines out $ ["module Main(main) where"] ++ ["import " ++ m | m <- mods] ++ ["main = return ()"]
 
-    obj "Success.txt" %> \out -> do
+    "Success.txt" %> \out -> do
+        putNormal . ("Checking documentation for:\n" ++) =<< readFile' "Files.lst"
         needModules
-        need [obj "Main.hs", obj "Paths_shake.hs"]
+        need ["Main.hs"]
+        trackAllow ["dist//*"]
         needSource
-        unit $ cmd "runhaskell -ignore-package=hashmap " ["-i" ++ obj "","-isrc",obj "Main.hs"]
+        cmd_ "ghc -fno-code -ignore-package=hashmap " ["-idist/build/autogen","-i" ++ root </> "src","Main.hs"]
         writeFile' out ""
 
 
@@ -158,7 +175,7 @@ findCodeHaddock src =
 findCodeMarkdown :: [String] -> [Code]
 findCodeMarkdown (x:xs) | indented x && not (isBlank x) =
     let (a,b) = span (\x -> indented x || isBlank x) (x:xs)
-    in Code (unindent a) : findCodeMarkdown b
+    in Code (dropWhileEnd isBlank $ unindent a) : findCodeMarkdown b
     where
         indented x = length (takeWhile isSpace x) >= 4
 findCodeMarkdown (x:xs) = map (Code . return) (evens $ splitOn "`" x) ++ findCodeMarkdown xs
@@ -176,18 +193,19 @@ showCode = concat . zipWith f [1..] . nubOrd
     where
         f i (Code x) | "#" `isPrefixOf` concat x = []
                      | all whitelist x = []
-                     | otherwise = showStmt i $ filter (not . isBlank . dropComment) $ map (fixCmd . undefDots) x
+                     | otherwise = showStmt i $ filter (not . isBlank . dropComment) $ fixCmd $ map undefDots x
 
 
-fixCmd :: String -> String
-fixCmd x | "cmd " `isPrefixOf` x || "command " `isPrefixOf` x = "unit $ " ++ x
-fixCmd x = replace "Stdout out" "Stdout (out :: String)" $ replace "Stderr err" "Stderr (err :: String)" x
+fixCmd :: [String] -> [String]
+fixCmd xs
+    | all ("cmd_ " `isPrefixOf`) xs = xs ++ ["return () :: IO () "]
+    | otherwise = map (replace "Stdout out" "Stdout (out :: String)" . replace "Stderr err" "Stderr (err :: String)") xs
 
 -- | Replace ... with undefined (don't use undefined with cmd; two ...'s should become one replacement)
 undefDots :: String -> String
 undefDots x | Just x <- stripSuffix "..." x, Just (x,_) <- stripInfix "..." x = x ++ new
             | otherwise = replace "..." new x
-    where new = if words x `disjoint` ["cmd","Development.Shake.cmd"] then "undefined" else "[\"\"]"
+    where new = if words x `disjoint` ["cmd","cmd_","Development.Shake.cmd","Development.Shake.cmd_"] then "undefined" else "[\"\"]"
 
 showStmt :: Int -> [String] -> [String]
 showStmt i [] = []
@@ -256,10 +274,11 @@ italics = words "command-name file-name N"
 types :: [String]
 types = words $
     "MVar IO String FilePath Maybe [String] Char ExitCode Change " ++
-    "Action Resource Assume FilePattern Development.Shake.FilePattern " ++
+    "Action Resource Rebuild FilePattern Development.Shake.FilePattern " ++
     "Lint Verbosity Rules CmdOption Int Double " ++
     "NFData Binary Hashable Eq Typeable Show Applicative " ++
-    "CmdResult ByteString ProcessHandle Rule Monad Monoid Data TypeRep"
+    "CmdResult ByteString ProcessHandle Rule Monad Monoid Data TypeRep " ++
+    "BuiltinRun BuiltinLint"
 
 -- | Duplicated identifiers which require renaming
 dupes :: [String]
@@ -267,7 +286,7 @@ dupes = words "main progressSimple rules"
 
 
 isFilePath :: String -> Bool
-isFilePath x = all validChar  x && ("foo/" `isPrefixOf` x || takeExtension x `elem` exts)
+isFilePath x = "C:\\" `isPrefixOf` x || (all validChar  x && ("foo/" `isPrefixOf` x || takeExtension x `elem` exts))
     where
         validChar x = isAlphaNum x || x `elem` "_./*"
         exts = words $ ".txt .hi .hs .o .exe .tar .cpp .cfg .dep .out .deps .m .h .c .html .zip " ++
@@ -275,7 +294,7 @@ isFilePath x = all validChar  x && ("foo/" `isPrefixOf` x || takeExtension x `el
 
 isCmdFlag :: String -> Bool
 isCmdFlag "+RTS" = True
-isCmdFlag x = length a >= 1 && length a <= 2 && all (\x -> isAlphaNum x || x `elem` "-=/_[]") b
+isCmdFlag x = length a `elem` [1,2] && all (\x -> isAlphaNum x || x `elem` "-=/_[]") b
     where (a,b) = span (== '-') x
 
 isCmdFlags :: String -> Bool
@@ -290,7 +309,7 @@ isEnvVar x | Just x <- stripPrefix "$" x = all validChar x
 
 isProgram :: String -> Bool
 isProgram (words -> x:xs) = x `elem` programs && all (\x -> isCmdFlag x || isFilePath x || all isAlpha x || x == "&&") xs
-    where programs = words "excel gcc cl make ghc cabal distcc build tar git fsatrace ninja touch pwd runhaskell rot13 main shake stack"
+    where programs = words "excel gcc cl make ghc ghci cabal distcc build tar git fsatrace ninja touch pwd runhaskell rot13 main shake stack rm cat sed sh apt-get build-multiple"
 
 -- | Should a fragment be whitelisted and not checked
 whitelist :: String -> Bool
@@ -298,7 +317,7 @@ whitelist x | null x || isFilePath x || isCmdFlags x || isEnvVar x || isProgram
 whitelist x | elem x $ words $
     "newtype do a q m c x value key os contents clean _make " ++
     ".. /. // \\ //* dir/*/* dir " ++
-    "ConstraintKinds TemplateHaskell GeneralizedNewtypeDeriving DeriveDataTypeable SetConsoleTitle " ++
+    "ConstraintKinds TemplateHaskell GeneralizedNewtypeDeriving DeriveDataTypeable TypeFamilies SetConsoleTitle " ++
     "Data.List System.Directory Development.Shake.FilePath run " ++
     "NoProgress Error src about://tracing " ++
     ".make/i586-linux-gcc/output build " ++
@@ -308,14 +327,16 @@ whitelist x | elem x $ words $
     "_shake _shake/build manual chrome://tracing/ compdb " ++
     "docs/manual foo.* _build _build/run depfile 0.000s " ++
     "@ndm_haskell file-name .PHONY filepath trim base stack extra #include " ++
-    "*> "
+    "*> BuiltinRun BuiltinLint RuleResult"
     = True
 whitelist x = x `elem`
     ["[Foo.hi, Foo.o]"
     ,"shake-progress"
+    ,"type instance"
     ,"1m25s (15%)"
     ,"3m12s (82%)"
     ,"getPkgVersion $ GhcPkgVersion \"shake\""
+    ,"ghc --make MyBuildSystem -threaded -rtsopts \"-with-rtsopts=-I0 -qg -qb\""
     ,"# command-name (for file-name)"
     ,"<i>build rules</i>"
     ,"<i>actions</i>"
@@ -326,7 +347,8 @@ whitelist x = x `elem`
     ,"#!/bin/sh"
     ,"shake-build-system"
     ,"\"_build\" </> x -<.> \"o\""
-    ,"cmd \"gcc -o\" [out] os"
     ,"[item1,item2,item2]"
     ,"$(LitE . StringL . loc_filename <$> location)"
+    ,"-d[ FILE], --debug[=FILE]"
+    ,"-r[ FILE], --report[=FILE], --profile[=FILE]"
     ]
