@@ -7,17 +7,19 @@ import Control.Exception.Extra
 import Control.Monad
 import Data.Either.Extra
 import System.Time.Extra
+import System.Info.Extra
 import Test.Type
 
 
-main = shaken test $ \args obj ->
+main = shakeTest_ test $
     action $ do
         putNormal "Starting sleep"
         liftIO $ sleep 5
         putNormal "Finished sleep"
 
 
-test build obj = do
+-- Disabled under Mac because it fails, see #560
+test build = unless isMac $ do
     -- check it fails exactly once
     time <- offsetTime
     lock <- newLock
@@ -31,6 +33,6 @@ test build obj = do
     b <- try_ b
     out "after try b"
     when (length (filter isLeft [a,b]) /= 1) $
-        error $ "Expected one success and one failure, got " ++ show [a,b]
+        fail $ "Expected one success and one failure, got " ++ show [a,b]
     -- check it succeeds after the lock has been held
     build []
