@@ -9,11 +9,11 @@ import Test.QuickCheck
 import Control.Monad
 import Data.List
 import qualified Data.ByteString.Char8 as BS
-import qualified Development.Shake.ByteString as BS
+import qualified Development.Shake.Internal.FileName as BS
 import System.Info.Extra
 
 
-main = shakenCwd test $ \args obj -> return ()
+main = shakeTest_ test $ return ()
 
 
 newtype File = File String deriving Show
@@ -23,7 +23,7 @@ instance Arbitrary File where
     shrink (File x) = map File $ shrink x
 
 
-test build obj = do
+test build = do
     let a === b = a Test.Type.=== b -- duplicate definition in QuickCheck 2.7 and above
 
     let norm x =
@@ -66,7 +66,7 @@ test build obj = do
         let y = norm x
             sep = Native.isPathSeparator
             noDrive = if isWindows then drop 1 else id
-            ps = [length y >= 1
+            ps = [y /= ""
                  ,null x || (sep (head x) == sep (head y) && sep (last x) == sep (last y))
                  ,not $ "/./" `isInfixOf` y
                  ,not isWindows || '\\' `notElem` y
