@@ -7,16 +7,16 @@ import Development.Shake.FilePath
 import Control.Monad.Extra
 import Test.Type
 
-main = shaken test $ \args obj -> forwardRule $ do
-    let src = "src/Test/C"
+main = shakeTest_ test $ forwardRule $ do
+    let src = root </> "src/Test/C"
     cs <- getDirectoryFiles src ["*.c"]
     os <- forP cs $ \c -> do
-        let o = obj c <.> "o"
+        let o = c <.> "o"
         cache $ cmd "gcc -c" [src </> c] "-o" [o]
         return o
-    cache $ cmd "gcc -o" [obj "Main" <.> exe] os
+    cache $ cmd "gcc -o" ["Main" <.> exe] os
 
-test build obj = do
+test build =
     whenM hasTracker $ do
         build ["--forward","--clean"]
         build ["--forward","-j2"]
