@@ -14,16 +14,15 @@ rebuilt :: IORef Int
 rebuilt = unsafePerformIO $ newIORef 0
 
 
-main = shakenCwd test $ \args obj -> do
-    want $ map obj ["a.out","b.out","c.out"]
-    obj "*.out" %> \out -> do
+main = shakeTest_ test $ do
+    want ["a.out","b.out","c.out"]
+    "*.out" %> \out -> do
         liftIO $ atomicModifyIORef rebuilt $ \a -> (a+1,())
         copyFile' (out -<.> "in") out
 
 
-test build obj = do
-    
-    let change x = writeFile (obj $ x <.> "in") x
+test build = do
+    let change x = writeFile (x <.> "in") x
     let count x = do
             before <- readIORef rebuilt
             build ["--sleep"]
