@@ -9,17 +9,20 @@ import Control.Concurrent.Extra
 import Data.IORef
 
 
-main = shakenCwd test $ \args obj -> do
-    want args
+main = shakeTest_ test $ do
+    "AB.txt" %> \out -> do
+        -- need [obj "A.txt", obj "B.txt"]
+        (text1,text2) <- readFile' "A.txt" `par` readFile' "B.txt"
+        writeFile' out $ text1 ++ text2
 
     phony "cancel" $ do
-        writeFile' (obj "cancel") ""
+        writeFile' "cancel" ""
         done <- liftIO $ newIORef 0
         lock <- liftIO newLock
         void $ parallel $ replicate 5 $ liftIO $ do
             x <- atomicModifyIORef done $ dupe . succ
             when (x == 3) $ do sleep 0.1; fail "boom"
-            withLock lock $ appendFile (obj "cancel") "x"
+            withLock lock $ appendFile "cancel" "x"
 
     phony "parallel" $ do
         active <- liftIO $ newIORef 0
@@ -30,13 +33,21 @@ main = shakenCwd test $ \args obj -> do
             sleep 0.1
             atomicModifyIORef active $ dupe . pred
         peak <- liftIO $ readIORef peak
-        writeFile' (obj "parallel") $ show peak
+        writeFile' "parallel" $ show peak
 
 
-test build obj = do
+test build = do
+    writeFile "A.txt" "AAA"
+    writeFile "B.txt" "BBB"
+    build ["AB.txt","--sleep"]
+    assertContents "AB.txt" "AAABBB"
+    appendFile "A.txt" "aaa"
+    build ["AB.txt"]
+    assertContents "AB.txt" "AAAaaaBBB"
+
     assertException ["boom"] $ build ["cancel","-j1","--quiet"]
-    assertContents (obj "cancel") "xx"
+    assertContents "cancel" "xx"
     build ["parallel","-j1"]
-    assertContents (obj "parallel") "1"
+    assertContents "parallel" "1"
     build ["parallel","-j5"]
-    assertContents (obj "parallel") "5"
+    assertContents "parallel" "5"
