@@ -10,6 +10,7 @@ import Control.Exception.Extra
 import Control.Monad
 import Data.List
 import Data.Maybe
+import General.GetOpt
 import System.Environment
 import System.Exit
 import System.Random
@@ -31,10 +32,11 @@ data Logic = Logic Int [[Source]]
            | Want [Int]
     deriving (Read,Show)
 
+arg = [Option "" ["arg"] (ReqArg Right "") ""]
 
-main = shaken test $ \args obj -> do
-    let toFile (Input i) = obj $ "input-" ++ show i ++ ".txt"
-        toFile (Output i) = obj $ "output-" ++ show i ++ ".txt"
+main = shakeTest test arg $ \args -> do
+    let toFile (Input i) = "input-" ++ show i ++ ".txt"
+        toFile (Output i) = "output-" ++ show i ++ ".txt"
         toFile Bang = error "BANG"
 
     let randomSleep = liftIO $ do
@@ -59,7 +61,7 @@ asDuration x
     | otherwise = Nothing
 
 
-test build obj = do
+test build = do
     limit <- do
         args <- getArgs
         let bound = listToMaybe $ reverse $ mapMaybe asDuration args
@@ -74,18 +76,18 @@ test build obj = do
         build ["clean"]
         build [] -- to create the directory
         forM_ inputRange $ \i ->
-            writeFile (obj $ "input-" ++ show i ++ ".txt") $ show $ Single i
+            writeFile ("input-" ++ show i ++ ".txt") $ show $ Single i
         logic <- randomLogic
         runLogic [] logic
         chng <- filterM (const randomIO) inputRange   
         forM_ chng $ \i ->
-            writeFile (obj $ "input-" ++ show i ++ ".txt") $ show $ Single $ negate i
+            writeFile ("input-" ++ show i ++ ".txt") $ show $ Single $ negate i
         runLogic chng logic
         forM_ inputRange $ \i ->
-            writeFile (obj $ "input-" ++ show i ++ ".txt") $ show $ Single i
+            writeFile ("input-" ++ show i ++ ".txt") $ show $ Single i
         logicBang <- addBang =<< addBang logic
         j <- randomRIO (1::Int,8)
-        res <- try_ $ build $ "--exception" : ("-j" ++ show j) : map show (logicBang ++ [Want [i | Logic i _ <- logicBang]])
+        res <- try_ $ build $ "--exception" : ("-j" ++ show j) : map ((++) "--arg=" . show) (logicBang ++ [Want [i | Logic i _ <- logicBang]])
         case res of
             Left err
                 | "BANG" `isInfixOf` show err -> return () -- error I expected
@@ -102,7 +104,7 @@ test build obj = do
                     replicateM i $ randomElem poss
                 sleepFileTime
                 j <- randomRIO (1::Int,8)
-                build $ ("-j" ++ show j) : map show (xs ++ map Want wants)
+                build $ ("-j" ++ show j) : map ((++) "--arg=" . show) (xs ++ map Want wants)
 
                 let value i = case [ys | Logic j ys <- xs, j == i] of
                         [ys] -> Multiple $ flip map ys $ map $ \i -> case i of
@@ -110,7 +112,7 @@ test build obj = do
                             Output i -> value i
                 forM_ (concat wants) $ \i -> do
                     let wanted = value i
-                    got <- fmap read $ IO.readFile' $ obj $ "output-" ++ show i ++ ".txt"
+                    got <- fmap read $ IO.readFile' $ "output-" ++ show i ++ ".txt"
                     when (wanted /= got) $
                         error $ "INCORRECT VALUE for " ++ show i
 
