@@ -1,4 +1,4 @@
-{-# LANGUAGE GeneralizedNewtypeDeriving, DeriveDataTypeable #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving, DeriveDataTypeable, TypeFamilies #-}
 
 module Test.Self(main) where
 
@@ -19,10 +19,12 @@ import Prelude
 newtype GhcPkg = GhcPkg () deriving (Show,Typeable,Eq,Hashable,Binary,NFData)
 newtype GhcFlags = GhcFlags () deriving (Show,Typeable,Eq,Hashable,Binary,NFData)
 
+type instance RuleResult GhcPkg = [String]
+type instance RuleResult GhcFlags = [String]
 
-main = shaken noTest $ \args obj -> do
-    let moduleToFile ext xs = map (\x -> if x == '.' then '/' else x) xs <.> ext
-    want $ if null args then [obj "Main" <.> exe] else args
+main = shakeTest_ noTest $ do
+    let moduleToFile ext xs = replace "." "/" xs <.> ext
+    want ["Main" <.> exe]
 
     -- fixup to cope with Cabal's generated files
     let fixPaths x = if x == "Paths_shake.hs" then "Paths.hs" else x
@@ -31,9 +33,8 @@ main = shaken noTest $ \args obj -> do
         Stdout out <- quietly $ cmd "ghc-pkg list --simple-output"
         return $ words out
 
-    ghcFlags <- addOracle $ \GhcFlags{} -> do
-        pkgs <- readFileLines $ obj ".pkgs"
-        return $ map ("-package=" ++) pkgs
+    ghcFlags <- addOracle $ \GhcFlags{} ->
+        map ("-package=" ++) <$> readFileLines ".pkgs"
 
     let ghc args = do
             -- since ghc-pkg includes the ghc package, it changes if the version does
@@ -41,35 +42,35 @@ main = shaken noTest $ \args obj -> do
             flags <- ghcFlags $ GhcFlags ()
             cmd "ghc" flags args
 
-    obj "Main" <.> exe %> \out -> do
-        src <- readFileLines $ obj "Run.deps"
-        let os = map (obj . moduleToFile "o") $ "Run" : src
+    "Main" <.> exe %> \out -> do
+        src <- readFileLines "Run.deps"
+        let os = map (moduleToFile "o") $ "Run" : src
         need os
         ghc $ ["-o",out] ++ os
 
-    obj "//*.deps" %> \out -> do
+    "//*.deps" %> \out -> do
         dep <- readFileLines $ out -<.> "dep"
-        let xs = map (obj . moduleToFile "deps") dep
+        let xs = map (moduleToFile "deps") dep
         need xs
         ds <- nubOrd . sort . (++) dep <$> concatMapM readFileLines xs
         writeFileLines out ds
 
-    obj "//*.dep" %> \out -> do
-        src <- readFile' $ "src" </> fixPaths (unobj $ out -<.> "hs")
+    "//*.dep" %> \out -> do
+        src <- readFile' $ root </> "src" </> fixPaths (out -<.> "hs")
         let xs = hsImports src
-        xs <- filterM (doesFileExist . ("src" </>) . fixPaths . moduleToFile "hs") xs
+        xs <- filterM (doesFileExist . (\x -> root </> "src" </> x) . fixPaths . moduleToFile "hs") xs
         writeFileLines out xs
 
-    [obj "//*.o",obj "//*.hi"] &%> \[out,_] -> do
+    ["//*.o","//*.hi"] &%> \[out,_] -> do
         deps <- readFileLines $ out -<.> "deps"
-        let hs = "src" </> fixPaths (unobj $ out -<.> "hs")
-        need $ hs : map (obj . moduleToFile "hi") deps
-        ghc $ ["-c",hs,"-isrc","-main-is","Run.main"
-              ,"-hide-all-packages","-odir=output/self","-hidir=output/self","-i=output/self"] ++
-              ["-DPORTABLE","-fwarn-unused-imports"] -- to test one CPP branch
+        let hs = root </> "src" </> fixPaths (out -<.> "hs")
+        need $ hs : map (moduleToFile "hi") deps
+        ghc ["-c",hs,"-i" ++ root </> "src","-main-is","Run.main"
+            ,"-hide-all-packages","-outputdir=."
+            ,"-DPORTABLE","-fwarn-unused-imports"] -- to test one CPP branch
 
-    obj ".pkgs" %> \out -> do
-        src <- readFile' "shake.cabal"
+    ".pkgs" %> \out -> do
+        src <- readFile' $ root </> "shake.cabal"
         writeFileLines out $ sort $ cabalBuildDepends src
 
 
@@ -86,6 +87,7 @@ cabalBuildDepends :: String -> [String]
 cabalBuildDepends _ = packages ++ ["unix" | os /= "mingw32"]
 
 packages = words
-    ("base transformers binary unordered-containers hashable time bytestring " ++
+    ("base transformers binary unordered-containers hashable time bytestring primitive " ++
      "filepath directory process deepseq random utf8-string extra js-jquery js-flot") ++
-    ["old-time" | compilerVersion < makeVersion [7,6]]
+    ["old-time" | compilerVersion < makeVersion [7,6]] ++
+    ["semigroups" | compilerVersion < makeVersion [8,0]]
