@@ -11,17 +11,17 @@ import Data.Maybe
 import Prelude
 
 
-main = shaken noTest $ \args obj -> do
+main = shakeTest_ noTest $ do
     -- Example inspired by http://gittup.org/tup/ex_multiple_directories.html
-    usingConfigFile "src/Test/Tup/root.cfg"
+    usingConfigFile $ root </> "src/Test/Tup/root.cfg"
 
     action $ do
         keys <- getConfigKeys
-        need [obj $ x -<.> exe | x <- keys, takeExtension x == ".exe"]
+        need [x -<.> exe | x <- keys, takeExtension x == ".exe"]
 
     let objects dir key = do
-            let f x | takeExtension x == ".c" = obj $ dir </> x -<.> "o"
-                    | takeExtension x == ".a" = obj $ takeBaseName x </> "lib" ++ x
+            let f x | takeExtension x == ".c" = dir </> x -<.> "o"
+                    | takeExtension x == ".a" = takeBaseName x </> "lib" ++ x
                     | otherwise = error $ "Unknown extension, " ++ x
             x <- fromMaybe (error $ "Missing config key, " ++ key) <$> getConfig key
             return $ map f $ words x
@@ -31,13 +31,13 @@ main = shaken noTest $ \args obj -> do
         need os
         cmd "gcc" os "-o" [out]
 
-    obj "//lib*.a" %> \out -> do
+    "//lib*.a" %> \out -> do
         os <- objects (drop 3 $ takeBaseName out) $ drop 3 $ takeFileName out
         need os
         cmd "ar crs" [out] os
 
-    obj "//*.o" %> \out -> do
-        let src = "src/Test/Tup" </> unobj out -<.> "c"
+    "//*.o" %> \out -> do
+        let src = root </> "src/Test/Tup" </> out -<.> "c"
         need [src]
-        () <- cmd "gcc -c -MMD -MF" [out -<.> "d"] [src] "-o" [out] "-O2 -Wall -Isrc/Test/Tup/newmath"
+        cmd_ "gcc -c -MMD -MF" [out -<.> "d"] [src] "-o" [out] "-O2 -Wall" ["-I" ++ root </> "src/Test/Tup/newmath"]
         neededMakefileDependencies $ out -<.> "d"
